;Copyright (C) 1990 by J. Scott Sanbar.  All rights reserved.
;
;NonStopISR - an external assembler routine written for Turbo Pascal
;Once installed with Keyboard Interrupt 09h pointing to it, it will
;check for CTRL-ALT-DEL, CTRL-BREAK and CTRL-C, throwing them away
;if the corresponding flag is set in the Turbo Pascal program.
;Needs sister module NONSTOP.PAS

        BIOSData      equ 0040h            ;Bios data segment
        KB_Flag       equ es:[17h]         ;Shift status flag

        AltPad        equ es:[19h]         ;Alt-Keypad work area

        Port_a        equ 60h              ;Keyboard input
        Port_b        equ 61h              ;Keyboard output for clear/enable

        Alt_Shift     equ 08h              ;bit mask for alt key status
        Ctl_Shift     equ 04h              ;bit mask for control key status

        GetIntr09     equ 3509h            ;ax for DOS Get Intr 09 vector
        PutIntr09     equ 2509h            ;ax for DOS Put Intr 09 vector

        BreakMake     equ 046h             ;Make code for CTRL-BREAK
        TwoMake       equ 03h              ;Make code for regular 2 key
        AltBreak      equ 0B8h             ;Break code for Alt key

DATA    SEGMENT WORD PUBLIC
        ASSUME DS:DATA
EXTRN   NoBoot        : BYTE
EXTRN   NoBreak       : BYTE
EXTRN   NoCtrlC       : BYTE
EXTRN   NoCtrl2       : BYTE
EXTRN   NoAlt3        : BYTE

        OldIntr09     dd ?                 ;original Intr 09 vector
        NonStopISRPtr dd ?                 ;far pointer to my Intr 09 ISR

DATA    ENDS

CODE    SEGMENT BYTE PUBLIC
        ASSUME CS:CODE

InstallVector PROC NEAR
        PUBLIC InstallVector

        push ds                        ;save ds
        mov word ptr [NonStopISRPtr], offset NonStopISR;Store ISR address
        mov word ptr [NonStopISRPtr + 2], seg NonStopISR;
        mov ax, GetIntr09              ;DOS Function 35h - Get Vector
        int 21h
        mov word ptr [OldIntr09 + 2], es;save original int 9 vector
        mov word ptr [OldIntr09], bx   ;in OldIntr09
        mov dx, word ptr [NonStopISRPtr];put pointer to NonStopISR
        mov ax, word ptr [NonStopISRPtr + 2];in Int 09 Vector
        mov ds, ax
        mov ax, PutIntr09              ;DOS Function 25h - Set Vector
        int 21h
        pop ds                         ;restore ds
        ret

InstallVector ENDP

RemoveVector PROC NEAR
        PUBLIC RemoveVector

        push ds                         ;save ds
        mov dx, word ptr [OldIntr09]    ;restore old Int 9 vector
        mov ax, word ptr [OldIntr09 + 2]
        mov ds, ax
        mov ax, PutIntr09
        int 21h
        pop ds
        ret

RemoveVector ENDP

NonStopISR PROC
        PUBLIC NonStopISR

        sub     sp, 4                  ;make room for RETF cs:ip
        push    bp                     ;create "twisted" stack frame
        mov     bp,sp
        push    ds                     ;save used registers
        push    es
        push    ax
        mov     ax, BIOSData           ;point es
        mov     es, ax                 ;to BIOS data area
        mov     ax, seg DATA           ;point ds
        mov     ds, ax                 ;to data segment
        in      al, 60h                ;put make/break code in al
        xor     ah, ah                 ;zero ah
        or      ah, NoAlt3             ;is Alt-3 allowed?
        jz      LeavePad               ;Yes? Continue on with rest
        cmp     al, AltBreak           ;is this an Alt release?
        jnz     LeavePad               ;No?  Continue
        cmp     byte ptr AltPad, 3     ;Yes? Is Keypad work area = ^C?
        jnz     LeavePad               ;No?  Continue
        mov     byte ptr AltPad, 0     ;Yes? Zero it.
LeavePad:
        mov     ah, KB_Flag                ;put keyboard shift flags in ax
        and     ah, Ctl_Shift              ;mask out everything but ctrl flag
        or      ah, 00000000b              ;see if zero
        jz      NormalKey                  ;chain on if ctrl not pressed
        xor     ah, ah                     ;zero ah
        or      ah, NoBoot                 ;Is flag set to disable soft boot?
        jz      CheckBreak                 ;No? Go check for break
        mov     ah, KB_Flag                ;get keyboard shift flags
        and     ah, Alt_Shift              ;mask out all but alt flag
        or      ah, 00000000b              ;is result zero?
        jz      CheckBreak                 ;go on to check for break
        cmp     al, 53h                    ;is it del make?
        jnz     CheckBreak                 ;no, chain on to old int 09
        jmp short TossIt                   ;Soft boot attempted - no dice!
CheckBreak:
        xor     ah, ah                     ;zero ah
        or      ah, NoBreak                ;Flag set to disable ctrl-break?
        jz      CheckCtrl2                 ;No? Go check for Ctrl-2
        cmp     al, BreakMake              ;is it Break make?
        jnz     CheckCtrl2                 ;No? Go check for Ctrl-2
        jmp short TossIt                   ;Ctrl-Break attempted - toss it!
CheckCtrl2:
        xor    ah, ah                      ;zero ah
        or     ah, NoCtrl2                 ;Flag set to disable ctrl-2?
        jz     CheckCtrlC                  ;No?  Go check for Ctrl-C
        cmp    al, TwoMake                 ;Is 2 pressed?
        jne    CheckCtrlC                  ;No?  Continue
        jmp short Tossit                   ;Ctrl-2 attempted - toss it!
CheckCtrlC:
        xor     ah, ah                     ;zero ah
        or      ah, NoCtrlC                ;flag set to disabe ctrl-c?
        jz      NormalKey                  ;No? Chain on to old ISR
        cmp     al, 2Eh                    ;C pressed?
        jnz     NormalKey                  ;No? Chain on to old ISR
TossIt:
        in      al, Port_b                 ;read keyboard control port
        mov     ah, al
        or      al, 10000000b              ;set the "reset" bit
        out     Port_b, al                 ;send it back to control
        xchg    ah, al                     ;get back control value
        out     Port_b, al                 ;send it out also
        cli
        mov     al, 20h                    ;send EOI to the
        out     20h, al                    ;interrupt controller
        pop     ax;                        ;cleanup
        pop     es;
        pop     ds;
        pop     bp;
        add     sp, 4                      ;get rid of spot for cs:ip
        iret                           ;LATER, DUDE!
NormalKey:
        sti                            ;allow interrupts
        push    [OldIntr09]            ;push old vector
        pop     [bp + 2]               ;put IP in stack for RETF
        pop     [bp + 4]               ;put CS in stack for RETF
        pop     ax                     ;cleanup and
        pop     es
        pop     ds
        pop     bp
        retf                           ;go to next link in daisy-chain

NonStopISR  ENDP
CODE    ENDS
        END     NonStopISR